package org.verohallinto.apitamoclient.ui;

import org.apache.commons.lang.exception.ExceptionUtils;
import org.apache.pivot.wtk.Form;
import org.apache.pivot.wtk.MessageType;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.verohallinto.apitamoclient.dto.ResponseDTO;
import org.verohallinto.apitamoclient.util.Const;
import org.verohallinto.apitamoclient.util.Utils;
import org.verohallinto.ilmoitin.gen.cxf.apitamopki.DeliveryDataRetrievalResponse;
import org.verohallinto.ilmoitin.gen.cxf.apitamopki.DeliveryDataSendResponse;
import org.verohallinto.ilmoitin.gen.cxf.apitamopki.LanguageType;

import java.io.File;
import java.io.FileNotFoundException;
import java.util.HashSet;
import java.util.Set;

/**
 *
 */
public class ApitamopkiClientUIAction {

    private static final Logger log = LoggerFactory.getLogger(ApitamopkiClientUIAction.class);
    private ApitamopkiClientUI ui;
    private String language;

    public ApitamopkiClientUIAction(final ApitamopkiClientUI ui) {

        this.ui = ui;
    }

    /**
     * Send material and attachments.
     */
    public void sendOperation() throws FileNotFoundException {

        if (log.isDebugEnabled()) {
            log.debug("sendOperation() - begin.");
        }

        Form.Flag flag = null;
        Form.setFlag(ui.materialBP, flag);
        Form.setFlag(ui.sendLanguageBP, flag);
        Form.setFlag(ui.emailBP, flag);
        Form.setFlag(ui.attachButtonBP, flag);
        ui.resFile.setText("");
        ui.status.setText("");
        setLanguage(ui.sendLanguageBG.getSelection().getButtonData().toString());
        setServiceInfo();

        final Set<String> attachments = new HashSet<>();

        if (!ui.attachLV.getListData().isEmpty()) {
            for (int x = 0; x < ui.attachLV.getListData().getLength(); x++) {
                if (ui.attachLV.isItemChecked(x)) {
                    attachments.add(((File)ui.attachLV.getListData().get(x)).getAbsolutePath());
                }
            }
        }

        final ResponseDTO dto = ui.apitamo.sendOperation(language,
                                                         ui.emails.getText(),
                                                         ui.backgroundCB.isSelected(),
                                                         ui.material.getText(),
                                                         attachments,
                                                         ui.sendResult.getText());

        if (Utils.isEmpty(dto.errorField)) {
            setSendResponse(dto);
            ui.tabs.setSelectedIndex(Const.TAB_RESPONSE);
        } else {
            flag = new Form.Flag(MessageType.ERROR);

            switch (dto.errorField) {
                case "materialBP":
                    Form.setFlag(ui.materialBP, flag);
                    Form.setFlag(ui.attachButtonBP, flag);
                    break;
                case "sendLanguageBP":
                    Form.setFlag(ui.sendLanguageBP, flag);
                    break;
                case "emailBP":
                    Form.setFlag(ui.emailBP, flag);
                    break;
                case "attachButtonBP":
                    Form.setFlag(ui.attachButtonBP, flag);
                    break;
                default:
                    break;
            }

            ui.status.setText(dto.sendResponse.getResult().getInformation());
        }

        if (log.isDebugEnabled()) {
            log.debug("sendOperation() - end.");
        }
    }

    /**
     * @param dto <code>ResponseDTO</code>
     */
    private void setSendResponse(final ResponseDTO dto) {

        if (log.isDebugEnabled()) {
            log.debug("setSendResponse() - begin.");
        }

        final StringBuilder sbLeft = new StringBuilder(256);
        final StringBuilder sbRight = new StringBuilder(256);
        final DeliveryDataSendResponse.Result result = dto.sendResponse.getResult();

        sbLeft.append("Aineisto").append(Const.SYS_LF);
        sbRight.append(result.getFiling()).append(Const.SYS_LF);
        // Aikaleima
        sbLeft.append("Aikaleima").append(Const.SYS_LF);
        sbRight.append(result.getTimestamp().toXMLFormat()).append(Const.SYS_LF);
        sbLeft.append("Hyvksytty").append(Const.SYS_LF);
        sbRight.append(result.isAccepted()).append(Const.SYS_LF);
        // TamoResult
        setTamoResult(dto, sbLeft, sbRight);

        if (!Utils.isEmpty(result.getCheckSum())) {
            // Tarkistussumma
            sbLeft.append("Tarkistussumma").append(Const.SYS_LF);
            sbRight.append(result.getCheckSum()).append(Const.SYS_LF);
        }

        if (!Utils.isEmpty(result.getDeliveryId())) {
            // Ilmoitustunniste
            sbLeft.append("Ilmoitustunniste").append(Const.SYS_LF);
            sbRight.append(result.getDeliveryId()).append(Const.SYS_LF);
        }

        if (!Utils.isEmpty(result.getRetrievalId())) {
            // Noutotunniste
            sbLeft.append("Noutotunniste").append(Const.SYS_LF);
            sbRight.append(result.getRetrievalId()).append(Const.SYS_LF);
        }

        if (!Utils.isEmpty(result.getResultId())) {
            // Tulostunniste
            sbLeft.append("Tulostunniste").append(Const.SYS_LF);
            sbRight.append(result.getResultId()).append(Const.SYS_LF);
        }

        if (!Utils.isEmpty(result.getInformation())) {
            // Info
            sbLeft.append("Info").append(Const.SYS_LF);
            sbRight.append(result.getInformation()).append(Const.SYS_LF);
        }

        if (result.getAttachments() != null && !result.getAttachments().getAttachment().isEmpty()) {
            // Liitteet
            sbLeft.append(Const.SYS_LF);
            sbRight.append(Const.SYS_LF);
            int x = 0;

            for (DeliveryDataSendResponse.Result.Attachments.Attachment att : result.getAttachments().getAttachment()) {
                if (x == 0) {
                    sbLeft.append("Liitteet").append(Const.SYS_LF);
                } else {
                    sbLeft.append(Const.SYS_LF);
                }

                sbRight.append(att.getFileName()).append(Const.SYS_LF);
                sbLeft.append(Const.SYS_LF);
                sbRight.append("    Hyvksytty").append(": ").append(att.isAccepted()).append(Const.SYS_LF);

                if (att.getInformation() != null && !att.getInformation().isEmpty()) {
                    sbLeft.append(Const.SYS_LF);
                    sbRight.append("    Info: ").append(att.getInformation()).append(Const.SYS_LF);
                }

                x++;
            }
        }

        ui.resLeft.setText(sbLeft.toString());
        ui.resRight.setText(sbRight.toString());

        // Aineiston checkupResult
        if (dto.result != null && dto.result.exists()) {
            ui.resFile.setText(dto.result.getAbsolutePath());
        }

        // Ilmoitusrivi
        ui.status.setText(dto.sendResponse.getResult().getInformation());

        if (log.isDebugEnabled()) {
            log.debug("setSendResponse() - end.");
        }
    }

    /**
     * Aineiston/tarkastustuloksen -nouto.
     */
    public void retrievalOperation() throws FileNotFoundException {

        if (log.isDebugEnabled()) {
            log.debug("retrievalOperation() - begin.");
        }

        Form.Flag flag = null;
        Form.setFlag(ui.retrBP, flag);
        Form.setFlag(ui.retrLanguageBP, flag);
        Form.setFlag(ui.retrIdBP, flag);
        ui.resFile.setText("");
        ui.status.setText("");
        setLanguage(ui.retrLanguageBG.getSelection().getButtonData().toString());
        setServiceInfo();

        if (Utils.isEmpty(ui.retrResult.getText())) {
            ui.retrResult.setText(getWorkDir().getAbsolutePath());
        }

        final String materialID = ui.retrMaterialRB.isSelected() ? ui.retrId.getText() : null;
        final String resultID = ui.retrResultRB.isSelected() ? ui.retrId.getText() : null;

        final ResponseDTO dto = ui.apitamo.retrievalOperation(language,
                                                              materialID,
                                                              resultID,
                                                              ui.retrResult.getText());

        if (Utils.isEmpty(dto.errorField)) {
            setRetrievalResponse(dto);
            ui.tabs.setSelectedIndex(Const.TAB_RESPONSE);
        } else {
            flag = new Form.Flag(MessageType.ERROR);

            switch (dto.errorField) {
                case "retrLanguageBP":
                    Form.setFlag(ui.retrLanguageBP, flag);
                    break;
                case "retrIdBP":
                    Form.setFlag(ui.retrIdBP, flag);
                    break;
                default:
                    break;
            }

            ui.status.setText(dto.retrResponse.getInformation());
        }

        if (log.isDebugEnabled()) {
            log.debug("retrievalOperation() - end.");
        }
    }

    /**
     * @param dto <code>ResponseDTO</code>
     */
    private void setRetrievalResponse(final ResponseDTO dto) {

        if (log.isDebugEnabled()) {
            log.debug("setRetrievalResponse() - begin.");
        }

        final StringBuilder sbLeft = new StringBuilder(256);
        final StringBuilder sbRight = new StringBuilder(256);
        final DeliveryDataRetrievalResponse result = dto.retrResponse;
        // Aineisto
        sbLeft.append("Aineisto").append(Const.SYS_LF);
        sbRight.append(result.getFiling()).append(Const.SYS_LF);
        // Aikaleima
        sbLeft.append("Aikaleima").append(Const.SYS_LF);
        sbRight.append(result.getTimestamp().toXMLFormat()).append(Const.SYS_LF);

        if (result.getStatus() != null) {
            sbLeft.append("Tila").append(Const.SYS_LF);
            sbRight.append(result.getStatus().value()).append(Const.SYS_LF);
        }

        if (!Utils.isEmpty(result.getRetrievalId())) {
            sbLeft.append("Ilmoitustunniste").append(Const.SYS_LF);
            sbRight.append(result.getRetrievalId()).append(Const.SYS_LF);
        }

        if (!Utils.isEmpty(result.getResultId())) {
            sbLeft.append("Tulostunniste").append(Const.SYS_LF);
            sbRight.append(result.getResultId()).append(Const.SYS_LF);
        }

        if (!Utils.isEmpty(result.getInformation())) {
            sbLeft.append("Info").append(Const.SYS_LF);
            sbRight.append(result.getInformation()).append(Const.SYS_LF);
            // Ilmoitusrivi
            ui.status.setText(result.getInformation());
        }

        // TamoResult
        setTamoResult(dto, sbLeft, sbRight);

        ui.resLeft.setText(sbLeft.toString());
        ui.resRight.setText(sbRight.toString());

        // Aineiston checkupResult
        if (dto.result != null && dto.result.exists()) {
            ui.resFile.setText(dto.result.getAbsolutePath());
        }

        if (log.isDebugEnabled()) {
            log.debug("setRetrievalResponse() - end.");
        }
    }

    /**
     * @param s <code>String</code>
     */
    private void setLanguage(final String s) {

        try {
            this.language = LanguageType.fromValue(Const.LANG.get(s)).value();
        } catch (IllegalArgumentException e) {
            ui.status.setText("Virheellinen kielikoodi.");
            this.language = "fi";
        }
    }

    /**
     * @throws FileNotFoundException WSDL was not found.
     */
    private void setServiceInfo() throws FileNotFoundException {

        ui.wsdl = Utils.getWsdl();
        ui.apitamo.setKsJks(ui.ksJks);
        ui.apitamo.setKsPwd(ui.ksPwd);
        ui.apitamo.setKsAlias(ui.ksAlias);
        ui.apitamo.setKsAliasPwd(ui.ksAliasPwd);
        //
        ui.apitamo.setTsJks(ui.tsJks);
        ui.apitamo.setTsPwd(ui.tsPwd);
        //
        ui.apitamo.setAcceptHost(ui.acceptHost);
        //
        ui.apitamo.setConnTimeout(ui.connTimeout);
        ui.apitamo.setReadTimeout(ui.readTimeout);
    }

    public File getWorkDir() {

        // Oletushakemistoiksi nykyinen tyhakemisto
        final File file = new File("SafeToDelete.tmp");
        return file.getAbsoluteFile().getParentFile();
    }

    /**
     * Tiedostoptteen palautus annetusta tiedostosta.
     * </p>
     *
     * @param file <code>String</code> ksiteltv aineisto.
     * @return Annetun tiedoston pte tai "NONE", jos ptett ei lydy.
     */
    public String getExtension(final String file) {

        String ext = null;

        if (file != null && !file.isEmpty()) {
            int x = file.lastIndexOf('.');

            if (x != -1) {
                ext = file.substring(x).toUpperCase();
            }
        }

        if (ext == null) {
            ext = "NONE";
        }

        return ext;
    }

    /**
     * @param e <code>Exception</code>
     */
    public void handleException(final Exception e) {

        log.error(ExceptionUtils.getFullStackTrace(e));
        ui.status.setText(e.getMessage() != null ? e.getMessage() : "");
    }

    /**
     * @param dto     <code>ResponseDTO</code>
     * @param sbLeft  <code>StringBuilder</code>
     * @param sbRight <code>StringBuilder</code>
     */
    private void setTamoResult(final ResponseDTO dto, final StringBuilder sbLeft, final StringBuilder sbRight) {

        if (dto.checkupResult != null) {
            sbLeft.append("Ilmoituksia").append(Const.SYS_LF);
            sbRight.append(dto.checkupResult.getReports()).append(Const.SYS_LF);

            if (dto.checkupResult.getCorrect() != null) {
                sbLeft.append("Oikeellisia").append(Const.SYS_LF);
                sbRight.append(dto.checkupResult.getCorrect()).append(Const.SYS_LF);
            }

            if (dto.checkupResult.getErrors() != null) {
                sbLeft.append("Virheellisi").append(Const.SYS_LF);
                sbRight.append(dto.checkupResult.getErrors()).append(Const.SYS_LF);
            }

            if (dto.checkupResult.getRemarks() != null) {
                sbLeft.append("Huomautuksia").append(Const.SYS_LF);
                sbRight.append(dto.checkupResult.getRemarks()).append(Const.SYS_LF);
            }
        }
    }
}
